@@ -2,7 +2,7 @@
 
 /*
  * created by Marc Ewing.
- * Currently maintained by Andrew G. Morgan <morgan@parc.power.net>
+ * Currently maintained by Andrew G. Morgan <morgan@linux.kernel.org>
  *
  * $Id: pam_handlers.c,v 1.17 1997/04/05 06:55:24 morgan Exp morgan $
  *
@@ -42,7 +42,7 @@ static void _pam_free_handlers_aux(struct handler **hp);
 
 static int _pam_add_handler(pam_handle_t *pamh
 		     , int must_fail, int other, int type
-		     , int control_flag, const char *mod_path
+		     , int *actions, const char *mod_path
 		     , int argc, char **argv, int argvlen);
 
 /* Values for module type */
@@ -68,7 +68,7 @@ static int _pam_parse_conf_file(pam_handle_t *pamh, FILE *f
 	char *tok, *nexttok=NULL;
 	const char *this_service;
 	const char *mod_path;
-	int module_type, control_flag;
+	int module_type, actions[_PAM_RETURN_VALUES];
 	int other;            /* set if module is for PAM_DEFAULT_SERVICE */
 	int res;              /* module added successfully? */
 	int must_fail=0;      /* a badly formatted line must fail when used */
@@ -92,6 +92,7 @@ static int _pam_parse_conf_file(pam_handle_t *pamh, FILE *f
 #endif /* PAM_READ_BOTH_CONFS */
 	other = !_pam_strCMP(this_service, PAM_DEFAULT_SERVICE);
 
+	/* accept "service name" or PAM_DEFAULT_SERVICE modules */
 	if (!_pam_strCMP(this_service, pamh->service_name) || other) {
 	    /* This is a service we are looking for */
 	    D(("_pam_init_handlers: Found PAM config entry for: %s"
@@ -116,28 +117,39 @@ static int _pam_parse_conf_file(pam_handle_t *pamh, FILE *f
 	    }
 	    D(("Using %s config entry: %s", must_fail?"BAD ":"", tok));
 
-	    /* accept "service name" or PAM_DEFAULT_SERVICE modules */
-	    
+	    /* reset the actions to .._UNDEF's -- this is so that
+               we can work out which entries are not yet set (for default). */
+	    {
+		int i;
+		for (i=0; i<_PAM_RETURN_VALUES;
+		     actions[i++] = _PAM_ACTION_UNDEF);
+	    }
 	    tok = _pam_StrTok(NULL, " \n\t", &nexttok);
 	    if (!_pam_strCMP("required", tok)) {
-		control_flag = PAM_F_REQUIRED;
 		D(("*PAM_F_REQUIRED*"));
+		actions[PAM_SUCCESS] = _PAM_ACTION_OK;
+		actions[PAM_NEW_AUTHTOK_REQD] = _PAM_ACTION_OK;
+		_pam_set_default_control(actions, _PAM_ACTION_BAD);
 	    } else if (!_pam_strCMP("requisite", tok)) {
-		control_flag = PAM_F_CRITICAL;
 		D(("*PAM_F_REQUISITE*"));
+		actions[PAM_SUCCESS] = _PAM_ACTION_OK;
+		actions[PAM_NEW_AUTHTOK_REQD] = _PAM_ACTION_OK;
+		_pam_set_default_control(actions, _PAM_ACTION_DIE);
 	    } else if (!_pam_strCMP("optional", tok)) {
-		control_flag = PAM_F_OPTIONAL;
 		D(("*PAM_F_OPTIONAL*"));
+		actions[PAM_SUCCESS] = _PAM_ACTION_OK;
+		actions[PAM_NEW_AUTHTOK_REQD] = _PAM_ACTION_OK;
+		_pam_set_default_control(actions, _PAM_ACTION_IGNORE);
 	    } else if (!_pam_strCMP("sufficient", tok)) {
-		control_flag = PAM_F_SUFFICIENT;
 		D(("*PAM_F_SUFFICIENT*"));
+		actions[PAM_SUCCESS] = _PAM_ACTION_DONE;
+		actions[PAM_NEW_AUTHTOK_REQD] = _PAM_ACTION_DONE;
+		_pam_set_default_control(actions, _PAM_ACTION_IGNORE);
 	    } else {
-		/* Illegal flag */
-		D(("_pam_init_handlers: bad control flag: %s", tok));
-		_pam_log_error("(%s) illegal control flag: %s"
-			       , this_service, tok);
-		control_flag = PAM_F_REQUIRED;
-		must_fail = 1;             /* fail when we dispatch function */
+		D(("will need to parse %s", tok));
+		_pam_parse_control(actions, tok);
+		/* by default the default is to treat as failure */
+		_pam_set_default_control(actions, _PAM_ACTION_BAD);
 	    }
 
 	    tok = _pam_StrTok(NULL, " \n\t", &nexttok);
@@ -168,19 +180,27 @@ static int _pam_parse_conf_file(pam_handle_t *pamh, FILE *f
 	    {
 		int y;
 
-		D(("CONF%s: %s%s %d %d %s %d"
+		D(("CONF%s: %s%s %d %s %d"
 		   , must_fail?"<*will fail*>":""
 		   , this_service, other ? "(backup)":""
-		   , module_type, control_flag
+		   , module_type
 		   , mod_path, argc));
 		for (y = 0; y < argc; y++) {
 		    D(("CONF: %s", argv[y]));
 		}
+		for (y = 0; y<_PAM_RETURN_VALUES; ++y) {
+		    D(("RETURN %s(%d) -> %d %s",
+		       _pam_token_returns[y], y, actions[y],
+		       actions[y]>0 ? "jump":
+			_pam_token_actions[-actions[y]]));
+		}
+		fprintf(stderr, "pause to look at debugging: ");
+		getchar();
 	    }
 #endif
 
 	    res = _pam_add_handler(pamh, must_fail, other
-				   , module_type, control_flag, mod_path
+				   , module_type, actions, mod_path
 				   , argc, argv, argvlen);
 	    if (res != PAM_SUCCESS) {
 		_pam_log_error("error loading %s", mod_path);
@@ -451,7 +471,7 @@ typedef int (*servicefn)(pam_handle_t *, int, int, char **);
 
 int _pam_add_handler(pam_handle_t *pamh
 		     , int must_fail, int other, int type
-		     , int control_flag, const char *mod_path
+		     , int *actions, const char *mod_path
 		     , int argc, char **argv, int argvlen)
 {
     struct loaded_module *mod;
@@ -714,7 +734,7 @@ int _pam_add_handler(pam_handle_t *pamh
 
     (*handler_p)->must_fail = must_fail;        /* failure forced? */
     (*handler_p)->func = func;
-    (*handler_p)->control_flag = control_flag;
+    memcpy((*handler_p)->actions,actions,sizeof((*handler_p)->actions));
     (*handler_p)->argc = argc;
     (*handler_p)->argv = argv;                       /* not a copy */
     (*handler_p)->next = NULL;
@@ -733,7 +753,7 @@ int _pam_add_handler(pam_handle_t *pamh
 
 	(*handler_p2)->must_fail = must_fail;        /* failure forced? */
 	(*handler_p2)->func = func2;
-	(*handler_p2)->control_flag = control_flag;
+	memcpy((*handler_p2)->actions,actions,sizeof((*handler_p2)->actions));
 	(*handler_p2)->argc = argc;
 	if (argv) {
 	    if (((*handler_p2)->argv = malloc(argvlen)) == NULL) {
@@ -845,6 +865,7 @@ void _pam_free_handlers_aux(struct handler **hp)
 	last = h;
 	_pam_drop(h->argv);  /* This is all alocated in a single chunk */
 	h = h->next;
+	memset(last, 0, sizeof(*last));
 	free(last);
     }
 
