@@ -2,9 +2,12 @@
 
 /* Creator Marc Ewing
  *
- * $Id: pam_start.c,v 1.9 1997/02/15 16:02:31 morgan Exp $
+ * $Id: pam_start.c,v 1.10 1997/04/05 06:58:11 morgan Exp $
  *
  * $Log: pam_start.c,v $
+ * Revision 1.10  1997/04/05 06:58:11  morgan
+ * removed conditional compilation (fail-delay again)
+ *
  * Revision 1.9  1997/02/15 16:02:31  morgan
  * lower case service name forced
  *
@@ -101,9 +104,7 @@ int pam_start (
 	return PAM_ABORT;
     }
 
-#ifdef PAM_FAIL_DELAY_ON
     _pam_reset_timer(*pamh);         /* initialize timer support */
-#endif /* PAM_FAIL_DELAY_ON */
 
     _pam_start_handlers(*pamh);                   /* cannot fail */
 
