@@ -3,7 +3,7 @@
  *
  * A generic conversation function for text based applications
  *
- * Written by Andrew Morgan <morgan@parc.power.net>
+ * Written by Andrew Morgan <morgan@linux.kernel.org>
  *
  * $Log: misc_conv.c,v $
  * Revision 1.5  1997/01/04 20:16:48  morgan
@@ -23,8 +23,10 @@
  * Initial revision
  */
 
+#ifdef linux
 #define _GNU_SOURCE
 #include <features.h>
+#endif
 
 #include <signal.h>
 #include <stdio.h>
