@@ -1,7 +1,7 @@
 /* pam_nologin module */
 
 /*
- * $Id: pam_nologin.c,v 1.4 1997/04/05 06:36:47 morgan Exp $
+ * $Id: pam_nologin.c,v 1.4 1997/04/05 06:36:47 morgan Exp morgan $
  *
  * Written by Michael K. Johnson <johnsonm@redhat.com> 1996/10/24
  *
@@ -29,6 +29,7 @@
 #include <sys/stat.h>
 #include <pwd.h>
 
+#include <security/_pam_macros.h>
 /*
  * here, we make a definition for the externally accessible function
  * in this file (this definition is required for static a module
@@ -88,7 +89,9 @@ int pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc,
        pam_get_item(pamh, PAM_CONV, (const void **)&conversation);
        conversation->conv(1, (const struct pam_message **)&pmessage,
 			  &resp, conversation->appdata_ptr);
-       free(message.msg);
+       free(mtmp);
+       if (resp)
+	   _pam_drop_reply(resp, 1);
      }
 
      return retval;
