@@ -8,7 +8,7 @@
  * It is a little ugly to do it this way, but it is a simple way of
  * defining static functions only once, and yet keeping the separate
  * files modular. If you can think of something better, please email
- * Andrew Morgan <morgan@parc.power.net>
+ * Andrew Morgan <morgan@linux.kernel.org>
  *
  * See the end of this file for Copyright information.
  */
@@ -37,11 +37,13 @@
 
 static const char rcsid[] =
 "$Id: pam_pwdb.c,v 1.3 1997/01/04 20:38:33 morgan Exp morgan $\n"
-" - PWDB Pluggable Authentication module. <morgan@parc.power.net>"
+" - PWDB Pluggable Authentication module. <morgan@linux.kernel.org>"
 ;
 
-#define _GNU_SOURCE
-#include <features.h>
+#ifdef linux
+# define _GNU_SOURCE
+# include <features.h>
+#endif
 
 #include <sys/types.h>
 #include <stdarg.h>
