@@ -71,19 +71,19 @@ int innetgr(const char *, const char *, const char *,const char *);
 
 #include <net/if.h>
 #ifdef linux
-#include <linux/sockios.h>
+# include <linux/sockios.h>
+# ifndef __USE_MISC
+#  define __USE_MISC
+#  include <sys/fsuid.h>
+# endif /* __USE_MISC */
 #endif
 
-#ifndef __USE_MISC
-#define __USE_MISC
-#include <sys/fsuid.h>
-#endif /* __USE_MISC */
 #include <pwd.h>
 #include <grp.h>
 #include <sys/file.h>
 #include <sys/signal.h>
 #include <sys/stat.h>
-#include <sys/syslog.h>
+#include <syslog.h>
 #ifndef _PATH_HEQUIV
 #define _PATH_HEQUIV "/etc/hosts.equiv"
 #endif /* _PATH_HEQUIV */
@@ -339,12 +339,12 @@ __ivaliduser (pam_handle_t *pamh, struct _options *opts,
 {
     register const char *user;
     register char *p;
-    int ch, hcheck, ucheck;
+    int len, hcheck, ucheck;
     char buf[MAXHOSTNAMELEN + 128];                       /* host + login */
 
     buf[sizeof (buf)-1] = '\0';                 	/* terminate line */
 
-    while (fgets(buf, sizeof (buf)-1, hostf)) {   /* line from hostf file */
+    while ((len = fgets(buf, sizeof(buf), hostf))) {   /* hostf file line */
         p = buf;                              /* from beginning of file.. */
 
 	/* Skip empty or comment lines */
@@ -353,11 +353,11 @@ __ivaliduser (pam_handle_t *pamh, struct _options *opts,
 	}
 
 	/* Skip lines that are too long. */
-	if (strchr(p, '\n') == (char *) 0) {
-	    ch = getc(hostf);
+	if (len == sizeof(buf) && strchr(p, '\n') == NULL) {
+	    int ch = getc(hostf);
+
 	    while (ch != '\n' && ch != EOF)
 		ch = getc(hostf);
-
 	    continue;
 	}
 
